@@ -7,12 +7,14 @@ It's designed to be simple.
 A remote method is invoked by sending a request to a remote service.
 The request is a single object serialized using JSON.
 
-The specification is at http://groups.google.com/group/json-rpc/web/json-rpc-2-0
+The specification is at
+http://groups.google.com/group/json-rpc/web/json-rpc-2-0
 '''
+import sys
 import json
 from functools import partial
 
-from pulsar import async, Failure, multi_async
+from pulsar import async, Failure, multi_async, maybe_failure
 from pulsar.utils.structures import AttributeDictionary
 from pulsar.utils.security import gen_unique_id
 from pulsar.utils.pep import range
@@ -49,8 +51,9 @@ Design to comply with the `JSON-RPC 2.0`_ Specification.
     _json = JsonToolkit
 
     def __call__(self, request):
-        return Json(self._call(request), json=self._json).http_response(request)
-    
+        return Json(self._call(request),
+                    json=self._json).http_response(request)
+
     @async()
     def _call(self, request):
         response = request.response
@@ -60,16 +63,17 @@ Design to comply with the `JSON-RPC 2.0`_ Specification.
             response.content_type = 'application/json'
         try:
             try:
-                data = request.json_data
+                data = yield request.body_data()
             except ValueError:
-                raise InvalidRequest(status=415,
-                                msg='Content-Type must be application/json')
+                raise InvalidRequest(
+                    status=415, msg='Content-Type must be application/json')
             if response.content_type not in JSON_CONTENT_TYPES:
-                raise InvalidRequest(status=415,
-                                msg='Content-Type must be application/json')
+                raise InvalidRequest(
+                    status=415, msg='Content-Type must be application/json')
             if data.get('jsonrpc') != self.version:
                 raise InvalidRequest(
-                    'jsonrpc must be supplied and equal to "%s"' % self.version)
+                    'jsonrpc must be supplied and equal to "%s"' %
+                    self.version)
             params = data.get('params')
             if isinstance(params, dict):
                 args, kwargs = (), params
@@ -78,54 +82,60 @@ Design to comply with the `JSON-RPC 2.0`_ Specification.
             #
             callable = self.get_handler(data.get('method'))
             result = yield callable(request, *args, **kwargs)
-        except Exception as e:
-            result = Failure.make(e)
+        except Exception:
+            result = sys.exc_info()
         #
+        result = maybe_failure(result)
         res = {'id': data.get('id'), "jsonrpc": self.version}
         if isinstance(result, Failure):
             msg = None
             error = result.error
+            code = getattr(error, 'fault_code', -32603)
             if isinstance(error, TypeError):
                 msg = checkarity(callable, args, kwargs, discount=1)
             msg = msg or str(error) or 'JSON RPC exception'
-            result = {'code': getattr(error, 'fault_code', -32603),
+            if code == -32603:
+                result.log(msg=msg, level='error')
+            else:
+                result.log(msg=msg, level='warning')
+            result = {'code': code,
                       'message': msg,
-                      'data': getattr(error,'data','')}
+                      'data': getattr(error, 'data', '')}
             request.response.status_code = getattr(error, 'status', 500)
             res['error'] = result
         else:
             res['result'] = result
         yield res
-    
+
 
 class JsonCall:
-    
+
     dlots = ('_client', '_name')
-    
+
     def __init__(self, client, name):
         self._client = client
         self._name = name
-        
+
     def __repr__(self):
         return self._name
     __str__ = __repr__
-    
+
     @property
     def url(self):
         return self._client.url
-    
+
     @property
     def name(self):
         return self._name
-    
+
     def __getattr__(self, name):
         name = "%s%s%s" % (self._name, self._client.separator, name)
         return self.__class__(self._client, name)
-        
+
     def __call__(self, *args, **kwargs):
         return self._client._call(self._name, *args, **kwargs)
-    
-        
+
+
 class JsonProxy(object):
     '''A python Proxy class for :class:`JSONRPC` Servers.
 
@@ -151,13 +161,13 @@ Lets say your RPC server is running at ``http://domain.name.com/``::
     'pong'
 
 '''
-    separator  = '.'
-    rawprefix  = 'raw'
+    separator = '.'
     default_version = '2.0'
     default_timeout = 30
     _json = JsonToolkit
 
-    def __init__(self, url, version=None, data=None, full_response=False, **kw):
+    def __init__(self, url, version=None, data=None,
+                 full_response=False, **kw):
         self.__url = url
         self.__version = version or self.__class__.default_version
         self._full_response = full_response
@@ -169,6 +179,8 @@ Lets say your RPC server is running at ``http://domain.name.com/``::
         if not http:
             timeout = timeout if timeout is not None else self.default_timeout
             http = HttpClient(timeout=timeout, **kw)
+        http.headers['accept'] = 'application/json, text/*; q=0.5'
+        http.headers['content-type'] = 'application/json'
         self.local.http = http
 
     @property
@@ -178,7 +190,7 @@ Lets say your RPC server is running at ``http://domain.name.com/``::
     @property
     def http(self):
         return self.local.http
-    
+
     @property
     def version(self):
         return self.__version
@@ -192,7 +204,7 @@ Lets say your RPC server is running at ``http://domain.name.com/``::
 
     def __str__(self):
         return self.__repr__()
-    
+
     def __getattr__(self, name):
         return JsonCall(self, name)
 
@@ -209,44 +221,30 @@ usage is simple::
         return multi_async((func(*args, **kwargs) for t in range(times)))
 
     def _call(self, name, *args, **kwargs):
-        data, raw = self._get_data(name, *args, **kwargs)
+        data = self._get_data(name, *args, **kwargs)
         body = self._json.dumps(data).encode('utf-8')
-        # Always make sure the content-type is application/json
-        self.http.headers['content-type'] = 'application/json'
         resp = self.http.post(self.url, data=body)
         if self._full_response:
             return resp
-        elif hasattr(resp, 'on_finished'):
-            res = resp.on_finished.add_callback(partial(self._end_call, raw))
-            return res.result if self.http.force_sync else res
-        else:
-            return self._end_call(raw, resp)
-        
-    def _end_call(self, raw, resp):
-        content = resp.content_string()
+        res = resp.on_finished.add_callback(self._end_call)
+        return res.result if self.http.force_sync else res
+
+    def _end_call(self, resp):
+        content = resp.decode_content(object_hook=DefaultJSONHook)
         if resp.is_error:
             if 'error' in content:
                 return self.loads(content)
             else:
                 resp.raise_for_status()
         else:
-            if raw:
-                return content
-            else:
-                return self.loads(content)
-        
+            return self.loads(content)
+
     def _get_data(self, func_name, *args, **kwargs):
         id = self.makeid()
-        fs = func_name.split('_')
-        raw = False
-        if len(fs) > 1 and fs[0] == self.rawprefix:
-            raw = True
-            fs.pop(0)
-            func_name = '_'.join(fs)
         params = self.get_params(*args, **kwargs)
         data = {'method': func_name, 'params': params, 'id': id,
                 'jsonrpc': self.__version}
-        return data, raw
+        return data
 
     def get_params(self, *args, **kwargs):
         '''
@@ -263,11 +261,10 @@ usage is simple::
             return kwargs
 
     def loads(self, obj):
-        res = self._json.loads(obj)
-        if isinstance(res, dict):
-            if 'error' in res:
-                error = res['error']
+        if isinstance(obj, dict):
+            if 'error' in obj:
+                error = obj['error']
                 raise exception(error.get('code'), error.get('message'))
             else:
-                return res.get('result')
-        return res
+                return obj.get('result')
+        return obj
