@@ -9,13 +9,13 @@ This is a Web Socket handler which echos all received messages
 back to the client::
 
     from pulsar.apps import wsgi, ws
-    
+
     class EchoWS(ws.WS):
-    
+
         def on_message(self, websocket, message):
             websocket.write(message)
-            
-            
+
+
 To create a valid :class:`WebSocket` middleware initialise as follow::
 
     wm = ws.WebSocket('/bla', EchoWS())
@@ -48,7 +48,7 @@ WebSocket handler
 .. autoclass:: WS
    :members:
    :member-order: bysource
-   
+
 
 WebSocket protocol
 ~~~~~~~~~~~~~~~~~~~~
@@ -56,41 +56,47 @@ WebSocket protocol
 .. autoclass:: WebSocketProtocol
    :members:
    :member-order: bysource
+
 '''
 from .websocket import WebSocket, WebSocketProtocol
-from .xhr import SocketIO
 
 
 class WS(object):
-    '''A web socket handler for both servers and clients. It implements
-the asynchronous message passing for a :class:`WebSocketProtocol`.
-On the server, the communication is started by the
-:class:`WebSocket` middleware after a successful handshake.
- 
-Override :meth:`on_message` to handle incoming string messages,
-:meth:`on_bytes` to handle incoming ``bytes`` messages
-You can also override :meth:`on_open` and :meth:`on_close` to handle opened
-and closed connections.
-These methods accept as first parameter the
-:class:`WebSocketProtocol` created during the handshake.
-'''
+    '''A web socket handler for both servers and clients.
+
+    It implements the asynchronous message passing for a
+    :class:`WebSocketProtocol`.
+    On the server, the communication is started by the
+    :class:`WebSocket` middleware after a successful handshake.
+
+    Override :meth:`on_message` to handle incoming string messages and
+    :meth:`on_bytes` to handle incoming ``bytes`` messages.
+
+    You can also override :meth:`on_open` and :meth:`on_close` to perform
+    specific tasks when the websocket is opened or closed.
+
+    These methods accept as first parameter the
+    :class:`WebSocketProtocol` created during the handshake.
+    '''
     def on_open(self, websocket):
         """Invoked when a new WebSocket is opened."""
         pass
 
     def on_message(self, websocket, message):
         '''Handles incoming messages on the WebSocket.
-This method must be overloaded.'''
+
+        This method must be overloaded.
+        '''
         pass
-    
+
     def on_bytes(self, websocket, body):
         '''Handles incoming bytes.'''
         pass
-    
+
     def on_ping(self, websocket, body):
         '''Handle incoming ping ``Frame``.'''
-        websocket.write(self.pong(websocket))
-        
+        websocket.pong(body)
+
     def on_pong(self, websocket, body):
         '''Handle incoming pong ``Frame``.'''
         pass
@@ -98,12 +104,11 @@ This method must be overloaded.'''
     def on_close(self, websocket):
         """Invoked when the WebSocket is closed."""
         pass
-    
+
     def pong(self, websocket, body=None):
         '''Return a ``pong`` frame.'''
         return websocket.parser.pong(body)
-    
+
     def ping(self, websocket, body=None):
         '''Return a ``ping`` frame.'''
         return websocket.parser.ping(body)
-        
