@@ -28,7 +28,7 @@ For more information regarding WSGI check the pep3333_ specification.
 To run the application::
 
     python script.py
-    
+
 For available run options::
 
     python script.py --help
@@ -40,8 +40,8 @@ WSGI Server
 .. autoclass:: WSGIServer
    :members:
    :member-order: bysource
-   
-   
+
+
 .. _`WSGI 1.0.1`: http://www.python.org/dev/peps/pep-3333/
 """
 from functools import partial
@@ -57,27 +57,21 @@ from .wrappers import *
 from .server import *
 from .route import *
 from .handlers import *
-from .plugins import *
-
-
-class WsgiFactory(object):
-    
-    def __call__(self):
-        raise NotImplementedError
+from .routers import *
+from .auth import *
 
 
 class WSGIServer(SocketServer):
-    '''A specialised :class:`pulsar.apps.socket.SocketServer` for the HTTP
-protocol and WSGI compatible applications.'''
+    '''A wsgi :class:`pulsar.apps.socket.SocketServer`.'''
     name = 'wsgi'
-    cfg = pulsar.Config(apps=['socket', 'wsgi'])
+    cfg = pulsar.Config(apps=['socket', 'wsgi'],
+                        server_software=pulsar.SERVER_SOFTWARE)
 
     def protocol_consumer(self):
-        '''Build the :class:`pulsar.ProtocolConsumer` factory for this
-WSGI server. It uses the :class:`pulsar.apps.wsgi.server.HttpServerResponse`
-protocol consumer and the wsgi callable provided as parameter during
-initialisation.'''
-        callable = self.callable
-        if isinstance(callable, WsgiFactory):
-            callable = callable()
-        return partial(HttpServerResponse, callable, self.cfg)
+        '''Build the :class:`pulsar.ProtocolConsumer` factory.
+
+        It uses the :class:`pulsar.apps.wsgi.server.HttpServerResponse`
+        protocol consumer and the wsgi callable provided as parameter during
+        initialisation.'''
+        c = self.cfg
+        return partial(HttpServerResponse, self.callable, c, c.server_software)
