@@ -1,6 +1,20 @@
-'''This module implements several WSGI middleware which does not
+'''
+Middleware are functions or callable objects similar to
+:ref:`WSGI application handlers <wsgi-handlers>`
+with the only difference that they can return ``None``. Middleware can be used
+in conjunction with a :class:`WsgiHandler` or any other handler which iterate
+through a list of middleware in a similar way (for example django wsgi
+handler).
+
+Here we introduce the :class:`Router` and :class:`MediaRouter` to handle
+requests on given urls. Pulsar is shipped with
+:ref:`additional wsgi middleware <wsgi-additional-middleware>` for manipulating
+the environment before a client response is returned.
+
+
+This module implements several WSGI middleware which does not
 serve request but instead perform initialization and sanity checks.
-It also introduces the **Response middlewares** implemented as a subclass of 
+It also introduces the **Response middlewares** implemented as a subclass of
 :class:`ResponseMiddleware`. Response middlewares are used by the
 :class:`pulsar.apps.wsgi.wrappers.WsgiResponse` to modify/add headers and
 manipulate content.
@@ -10,7 +24,8 @@ manipulate content.
 Additional WSGI Middlewares
 ============================
 
-Several :ref:`wsgi middleware <wsgi-middleware>` useful in several applications.
+Several :ref:`wsgi middleware <wsgi-middleware>` useful in several
+applications.
 
 clean path
 ~~~~~~~~~~~~~~~~~~
@@ -41,13 +56,13 @@ Interface
 .. autoclass:: ResponseMiddleware
    :members:
    :member-order: bysource
-   
+
 GZip Middleware
 ~~~~~~~~~~~~~~~~~~~~~~
 .. autoclass:: GZipMiddleware
    :members:
    :member-order: bysource
-   
+
 '''
 import re
 from gzip import GzipFile
@@ -55,7 +70,7 @@ from gzip import GzipFile
 import pulsar
 from pulsar.utils.httpurl import BytesIO, parse_cookie
 
-from .plugins import parse_authorization_header
+from .auth import parse_authorization_header
 
 re_accepts_gzip = re.compile(r'\bgzip\b')
 
@@ -75,11 +90,12 @@ def is_streamed(content):
         return True
     return False
 
+
 def clean_path_middleware(environ, start_response):
     '''Clean url from double slashes and redirect if needed.'''
     path = environ['PATH_INFO']
     if path and '//' in path:
-        url = re.sub("/+" , '/', path)
+        url = re.sub("/+", '/', path)
         if not url.startswith('/'):
             url = '/%s' % url
         qs = environ['QUERY_STRING']
@@ -87,10 +103,14 @@ def clean_path_middleware(environ, start_response):
             url = '%s?%s' % (url, qs)
         raise pulsar.HttpRedirect(url)
 
+
 def cookies_middleware(environ, start_response):
-    '''Parse the ``HTTP_COOKIE`` key in the ``environ`` and set
-the new ``http.cookie`` key in ``environ`` with a dictionary of cookies
-obtained via the :func:`pulsar.utils.httpurl.parse_cookie` function.'''
+    '''Parse the ``HTTP_COOKIE`` key in ``environ``.
+
+    Set the new ``http.cookie`` key in ``environ`` with a dictionary
+    of cookies obtained via the :func:`pulsar.utils.httpurl.parse_cookie`
+    function.
+    '''
     c = environ.get('http.cookie')
     if not isinstance(c, dict):
         c = environ.get('HTTP_COOKIE', '')
@@ -102,11 +122,14 @@ obtained via the :func:`pulsar.utils.httpurl.parse_cookie` function.'''
             c = parse_cookie(c)
         environ['http.cookie'] = c
 
+
 def authorization_middleware(environ, start_response):
-    """Parse the ``HTTP_AUTHORIZATION`` key in the ``environ`` if available
-and set the ``http.authorization`` key in ``environ`` with the result
-obtained from :func:`pulsar.apps.wsgi.plugins.parse_authorization_header`
-function."""
+    '''Parse the ``HTTP_AUTHORIZATION`` key in the ``environ``.
+
+    If available, set the ``http.authorization`` key in ``environ`` with
+    the result obtained from
+    :func:`pulsar.apps.wsgi.auth.parse_authorization_header` function.
+    '''
     key = 'http.authorization'
     c = environ.get(key)
     if c is None:
@@ -114,6 +137,18 @@ function."""
         if code in environ:
             environ[key] = parse_authorization_header(environ[code])
 
+
+def wait_for_body_middleware(environ, start_response):
+    '''Use this middleware to wait for the full body.
+
+    This middleware wait for the full body to be received before letting
+    other middleware to be processed.
+
+    Useful when using synchronous web-frameworks.
+    '''
+    return environ['stream'].on_message_complete.add_callback(lambda s: None)
+
+
 #####################################################    RESPONSE MIDDLEWARE
 class ResponseMiddleware(object):
     '''Bas class for :class:`pulsar.apps.wsgi.wrappers.WsgiResponse`
@@ -179,7 +214,8 @@ http://jython.xhaus.com/http-compression-in-python-and-jython
                 # Avoid gzipping if we've already got a content-encoding.
                 if 'Content-Encoding' in headers:
                     return False
-                # MSIE have issues with gzipped response of various content types.
+                # MSIE have issues with gzipped response of various
+                # content types.
                 if "msie" in environ.get('HTTP_USER_AGENT', '').lower():
                     ctype = headers.get('Content-Type', '').lower()
                     if not ctype.startswith("text/") or "javascript" in ctype:
