@@ -3,20 +3,17 @@ import time
 import re
 import textwrap
 import logging
-import traceback
 from datetime import datetime, timedelta
-from inspect import istraceback
 from email.utils import formatdate
 
 
-import pulsar
 from pulsar.utils.structures import MultiValueDict
-from pulsar import Failure, get_actor
 from pulsar.utils.html import escape
 from pulsar.utils.pep import to_string
 from pulsar.utils.httpurl import (has_empty_content, REDIRECT_CODES, iteritems,
-                                  parse_qsl, HTTPError, parse_dict_header)
-                                 
+                                  parse_qsl, HTTPError, parse_dict_header,
+                                  JSON_CONTENT_TYPES)
+
 from .structures import Accept, RequestCacheControl
 from .content import Html
 
@@ -27,38 +24,39 @@ __all__ = ['handle_wsgi_error',
            'set_wsgi_request_class',
            'HOP_HEADERS']
 
-HOP_HEADERS = frozenset((
-        'connection',
-        'keep-alive',
-        'proxy-authenticate',
-        'proxy-authorization',
-        'te',
-        'trailers',
-        'transfer-encoding',
-        'upgrade',
-        'server',
-        'date',
-    ))
-
+DEFAULT_RESPONSE_CONTENT_TYPES = ('text/html', 'text/plain'
+                                  ) + JSON_CONTENT_TYPES
+HOP_HEADERS = frozenset(('connection',
+                         'keep-alive',
+                         'proxy-authenticate',
+                         'proxy-authorization',
+                         'te',
+                         'trailers',
+                         'transfer-encoding',
+                         'upgrade',
+                         'server',
+                         'date')
+                        )
 LOGGER = logging.getLogger('pulsar.wsgi')
+error_css = '''
+.pulsar-error {
+    width: 500px;
+    margin: 50px auto;
+}
+'''
 
 _RequestClass = None
 
+
 def wsgi_request(environ, app_handler=None, urlargs=None):
     global _RequestClass
     return _RequestClass(environ, app_handler=app_handler, urlargs=urlargs)
 
+
 def set_wsgi_request_class(RequestClass):
     global _RequestClass
     _RequestClass = RequestClass
-    
-    
-def wsgi_iterator(gen, encoding):
-    for data in gen:
-        if isinstance(data, bytes):
-            yield data
-        else:
-            yield data.encode(encoding)
+
 
 def cookie_date(epoch_seconds=None):
     """Formats the time to ensure compatibility with Netscape's cookie
@@ -75,8 +73,9 @@ def cookie_date(epoch_seconds=None):
     rfcdate = formatdate(epoch_seconds)
     return '%s-%s-%s GMT' % (rfcdate[:7], rfcdate[8:11], rfcdate[12:25])
 
+
 def set_cookie(cookies, key, value='', max_age=None, expires=None, path='/',
-                domain=None, secure=False, httponly=False):
+               domain=None, secure=False, httponly=False):
     '''Set a cookie key into the cookies dictionary *cookies*.'''
     cookies[key] = value
     if expires is not None:
@@ -105,15 +104,18 @@ def set_cookie(cookies, key, value='', max_age=None, expires=None, path='/',
     if httponly:
         cookies[key]['httponly'] = True
 
+
 _accept_re = re.compile(r'([^\s;,]+)(?:[^,]*?;\s*q=(\d*(?:\.\d+)?))?')
 
+
 def parse_accept_header(value, cls=None):
     """Parses an HTTP Accept-* header.  This does not implement a complete
     valid algorithm but one that supports at least value and quality
     extraction.
 
-    Returns a new :class:`Accept` object (basically a list of ``(value, quality)``
-    tuples sorted by the quality with some additional accessor methods).
+    Returns a new :class:`Accept` object (basically a list of
+    ``(value, quality)`` tuples sorted by the quality with some additional
+    accessor methods).
 
     The second parameter can be a subclass of :class:`Accept` that is created
     with the parsed values and returned.
@@ -137,6 +139,7 @@ def parse_accept_header(value, cls=None):
         result.append((match.group(1), quality))
     return cls(result)
 
+
 def parse_cache_control_header(value, on_update=None, cls=None):
     """Parse a cache control header.  The RFC differs between response and
     request cache control, this method does not.  It's your responsibility
@@ -161,39 +164,42 @@ def parse_cache_control_header(value, on_update=None, cls=None):
 def _gen_query(query_string, encoding):
     # keep_blank_values=True
     for key, value in parse_qsl((query_string or ''), True):
-        yield to_string(key, encoding, errors='replace'),\
-              to_string(value, encoding, errors='replace')
+        yield (to_string(key, encoding, errors='replace'),
+               to_string(value, encoding, errors='replace'))
+
 
 def query_dict(query_string, encoding='utf-8'):
     if query_string:
         return dict(MultiValueDict(_gen_query(query_string, encoding)).items())
     else:
         return {}
-    
-    
+
+
 error_messages = {
     500: 'An exception has occurred while evaluating your request.',
     404: 'Cannot find what you are looking for.'
 }
 
+
 def wsgi_error_msg(response, msg):
     if response.content_type == 'application/json':
         return json.dumps({'status': response.status_code,
                            'message': msg})
     else:
         return msg
-    
+
+
 class dump_environ(object):
     __slots__ = ('environ',)
-    
+
     def __init__(self, environ):
         self.environ = environ
-        
+
     def __str__(self):
         env = iteritems(self.environ)
         return '\n%s\n' % '\n'.join(('%s = %s' % (k, v) for k, v in env))
-    
-    
+
+
 def handle_wsgi_error(environ, failure):
     '''The default handler for errors while serving an Http requests.
 
@@ -209,40 +215,40 @@ def handle_wsgi_error(environ, failure):
     else:
         response.status_code = getattr(error, 'status', 500)
         response.headers.update(getattr(error, 'headers', None) or ())
-    path = ' @ path "%s"' % environ.get('PATH_INFO','/')
+    path = '@ path "%s"' % environ.get('PATH_INFO', '/')
     status = response.status_code
-    if not failure.logged:  #Log only if not previously logged
-        e = dump_environ(environ)
-        failure.logged = True
-        if status == 500:
-            LOGGER.critical('Unhandled exception during WSGI response %s.%s',
-                            path, e, exc_info=failure.exc_info)
-        else:
-            LOGGER.warning('WSGI %s status code %s.', status, path)
-            LOGGER.debug('%s', e, exc_info=failure.exc_info)
+    if status == 500:
+        failure.log(msg='Unhandled exception during WSGI response %s.%s' %
+                    (path, dump_environ(environ)), level='critical')
+    else:
+        failure.log(msg='WSGI %s status code %s' % (status, path),
+                    level='warning')
     if has_empty_content(status, request.method) or status in REDIRECT_CODES:
         content = None
     else:
         request.cache.pop('html_document', None)
         renderer = environ.get('error.handler') or render_error
         try:
-            content = renderer(request, failure.exc_info)
-            if isinstance(content, Failure):
-                content.log()
-                content = None
+            content = renderer(request, failure)
         except Exception:
-            LOGGER.critical('Error while rendering error', exc_info=True)
+            LOGGER.critical('Error while rendering error')
             content = None
     response.content = content
     return response
-           
-def render_error(request, exc_info): 
-    debug = get_actor().cfg.debug
+
+
+def render_error(request, failure):
+    '''Default renderer for errors.'''
+    cfg = request.get('pulsar.cfg')
+    debug = cfg.debug if cfg else False
     response = request.response
+    if not response.content_type:
+        response.content_type = request.content_types.best_match(
+            DEFAULT_RESPONSE_CONTENT_TYPES)
     if response.content_type == 'text/html':
         request.html_document.head.title = response.status
     if debug:
-        msg = render_error_debug(request, exc_info)
+        msg = render_error_debug(request, failure)
     else:
         msg = error_messages.get(response.status_code) or ''
         if response.content_type == 'text/html':
@@ -251,40 +257,34 @@ def render_error(request, exc_info):
                 {0[msg]}
                 <h3>{0[version]}</h3>
             """).format({"reason": response.status, "msg": msg,
-                         "version": pulsar.SERVER_SOFTWARE})
+                         "version": request.environ['SERVER_SOFTWARE']})
     #
     if response.content_type == 'text/html':
-        request.html_document.body.append(msg)
-        return request.html_document.render(request)
+        doc = request.html_document
+        doc.head.embedded_css.append(error_css)
+        doc.body.append(Html('div', msg, cn='pulsar-error'))
+        return doc.render(request)
     else:
         return wsgi_error_msg(response, msg)
 
-def render_error_debug(request, exc_info):
+
+def render_error_debug(request, failure):
     '''Render the traceback into the content type in *response*.'''
-    if exc_info:
-        response = request.response
-        trace = exc_info[2]
-        is_html = response.content_type == 'text/html'
-        if istraceback(trace):
-            trace = traceback.format_exception(*exc_info)
-        if is_html:
-            error = Html('div', cn='section traceback error')
-        else:
-            error = []
-        if trace:
-            for traces in trace:
-                counter = 0
-                for trace in traces.split('\n'):
-                    if trace.startswith('  '):
-                        counter += 1
-                        trace = trace[2:]
-                    if not trace:
-                        continue
-                    if is_html:
-                        trace = Html('p', escape(trace))
-                        if counter:
-                            trace.css({'margin-left':'%spx' % (20*counter)})
-                    error.append(trace)
-        if not is_html:
-            error = '\n'.join(error)
-        return error
\ No newline at end of file
+    response = request.response
+    is_html = response.content_type == 'text/html'
+    error = Html('div', cn='section traceback error') if is_html else []
+    for traces in failure.exc_info[2]:
+        counter = 0
+        for trace in traces.split('\n'):
+            if trace.startswith('  '):
+                counter += 1
+                trace = trace[2:]
+            if trace:
+                if is_html:
+                    trace = Html('p', escape(trace))
+                    if counter:
+                        trace.css({'margin-left': '%spx' % (20*counter)})
+                error.append(trace)
+    if not is_html:
+        error = '\n'.join(error)
+    return error
