@@ -9,13 +9,6 @@ Config
    :members:
    :member-order: bysource
 
-SettingBase
-~~~~~~~~~~~~~~
-
-.. autoclass:: SettingBase
-   :members:
-   :member-order: bysource
-      
 Setting
 ~~~~~~~~~~
 
@@ -31,46 +24,30 @@ import inspect
 import argparse
 import os
 import textwrap
-import types
 import logging
 
-from pulsar import __version__, SERVER_NAME, parse_address
+from pulsar import __version__, SERVER_NAME
 from . import system
-from .httpurl import to_bytes, iteritems, HttpParser as PyHttpParser,\
-                     native_str
+from .internet import parse_address
 from .importer import import_system_file
+from .httpurl import HttpParser as PyHttpParser
+from .pep import to_bytes, iteritems, native_str
 
 
 __all__ = ['Config',
-           'SimpleSetting',
            'Setting',
-           #'defaults',
            'ordered_settings',
            'validate_string',
            'validate_callable',
            'validate_bool',
            'validate_list',
+           'validate_dict',
            'validate_pos_int',
            'validate_pos_float',
            'make_optparse_options']
 
 LOGGER = logging.getLogger('pulsar.config')
 
-class DefaultSettings:
-
-    def __init__(self):
-        # port for serving a socket
-        self.PORT = 8060
-        # timeout for asynchronous Input/Output
-        self.IO_TIMEOUT = None
-        # Maximum number of concurrenct clients
-        self.BACKLOG = 2048
-        # Actors timeout
-        self.TIMEOUT = 30
-        # mailbox clients blocking
-        self.message_timeout = 3
-
-defaults = DefaultSettings()
 section_docs = {}
 KNOWN_SETTINGS = {}
 KNOWN_SETTINGS_ORDER = []
@@ -78,16 +55,23 @@ KNOWN_SETTINGS_ORDER = []
 
 def pass_through(arg):
     '''A dummy function accepting one parameter only.
-    
+
 It does nothing and it is used as default by
 :ref:`Application Hooks <setting-section-application-hooks>`.'''
     pass
 
+
+def set_if_avail(container, key, value, *skip_values):
+    if value is not None and value not in skip_values:
+        container[key] = value
+
+
 def wrap_method(func):
     def _wrapped(instance, *args, **kwargs):
         return func(*args, **kwargs)
     return _wrapped
 
+
 def ordered_settings():
     for name in KNOWN_SETTINGS_ORDER:
         yield KNOWN_SETTINGS[name]
@@ -112,14 +96,14 @@ attribute by exposing the :attr:`Setting.name` as attribute.
     Dictionary of all :class:`Setting` instances available in this
     :class:`Config` container. Keys are given by the :attr:`SettingBase.name`
     attribute.
-    
+
 .. attribute:: params
 
     Dictionary of additional parameters which cannot be parsed in the
     command line.
 '''
     exclude_from_config = set(('config',))
-    
+
     def __init__(self, description=None, epilog=None,
                  version=None, apps=None, include=None,
                  exclude=None, settings=None, prefix=None,
@@ -140,20 +124,20 @@ attribute by exposing the :attr:`Setting.name` as attribute.
         self.epilog = epilog or 'Have fun!'
         self.version = version or __version__
         self.update(params)
-        
+
     def __iter__(self):
         return iter(self.settings)
-    
+
     def __len__(self):
         return len(self.settings)
-    
+
     def __contains__(self, name):
         return name in self.settings
-    
+
     def items(self):
         for k, setting in iteritems(self.settings):
             yield k, setting.value
-    
+
     def __getstate__(self):
         return self.__dict__.copy()
 
@@ -175,14 +159,14 @@ attribute by exposing the :attr:`Setting.name` as attribute.
         if name != "settings" and name in self.settings:
             raise AttributeError("Invalid access!")
         super(Config, self).__setattr__(name, value)
-    
+
     def update(self, data):
         '''Update this :attr:`Config` with ``data`` which is either an
 instance of Mapping or :class:`Config`.'''
         for name, value in data.items():
             if value is not None:
                 self.set(name, value)
-            
+
     def get(self, name, default=None):
         '''Get the value at ``name`` for this :class:`Config` container
 following this algorithm:
@@ -205,7 +189,7 @@ also set.'''
             if self.prefix:
                 prefix_name = '%s_%s' % (self.prefix, name)
                 if prefix_name in self.settings:
-                    return # dont set this value
+                    return  # don't set this value
             self.params[name] = value
         else:
             self.settings[name].set(value, default=default)
@@ -232,14 +216,14 @@ settings via the :meth:`Setting.add_argument` method.
         for k in sorted(setts, key=sorter):
             setts[k].add_argument(parser)
         return parser
-        
+
     def import_from_module(self, mod=None):
         if mod:
             self.set('config', mod)
         try:
             mod = import_system_file(self.config)
         except Exception as e:
-            raise RuntimeError('Failed to read config file "%s". %s' %\
+            raise RuntimeError('Failed to read config file "%s". %s' %
                                (self.config, e))
         unknowns = []
         if mod:
@@ -255,7 +239,7 @@ settings via the :meth:`Setting.add_argument` method.
                 else:
                     self.set(kl, val)
         return unknowns
-        
+
     def on_start(self):
         '''Invoked by a :class:`pulsar.Application` just before starting.'''
         for sett in self.settings.values():
@@ -297,7 +281,7 @@ container.'''
             pn = self.settings.get('default_process_name')
             if pn:
                 return pn.get()
-        
+
     def copy(self, name=None, prefix=None):
         '''A deep copy of this :class:`Config` container. If ``prefix``
 is given, it prefixes all non
@@ -316,19 +300,19 @@ is given, it prefixes all non
             me.settings[setting.name] = setting
         me.params = me.params.copy()
         return me
-    
+
     def __copy__(self):
         return self.copy()
-    
+
     def __deepcopy__(self, memo):
         return self.__copy__()
-    
-    ############################################################################
+
+    ########################################################################
     ##    INTERNALS
     def update_settings(self):
         for s in ordered_settings():
             setting = s().copy(name=self.name, prefix=self.prefix)
-            if setting.name in self.settings:  
+            if setting.name in self.settings:
                 continue
             if setting.name not in self.include:
                 if setting.name in self.exclude:
@@ -336,7 +320,7 @@ is given, it prefixes all non
                 if setting.app and setting.app not in self.apps:
                     continue    # the setting is for an app not in the apps set
             self.settings[setting.name] = setting
-            
+
     def _get(self, name, default=None):
         if name not in self.settings:
             if name in KNOWN_SETTINGS:
@@ -345,15 +329,14 @@ is given, it prefixes all non
                 return self.params[name]
             raise KeyError("'%s'" % name)
         return self.settings[name].get()
-    
-            
+
 
 class SettingMeta(type):
     '''A metaclass which collects all setting classes and put them
 in the global ``KNOWN_SETTINGS`` list.'''
     def __new__(cls, name, bases, attrs):
         super_new = super(SettingMeta, cls).__new__
-        parents = [b for b in bases if isinstance(b, SettingMeta)]
+        #parents = [b for b in bases if isinstance(b, SettingMeta)]
         val = attrs.get("validator")
         attrs["validator"] = wrap_method(val) if val else None
         if attrs.pop('virtual', False):
@@ -380,74 +363,32 @@ in the global ``KNOWN_SETTINGS`` list.'''
         setattr(cls, "short", '' if not lines else lines[0])
 
 
-class SettingBase(object):
-    '''This is the base class of :class:`Settings` and :class:`SimpleSetting`.
-It defines attributes for a given setting parameter.'''
+# This works for Python 2 and Python 3
+class Setting(SettingMeta('BaseSettings', (object,), {'virtual': True})):
+    '''A configuration parameter for pulsar.
+
+    Parameters can be specified on the command line or on a config file.
+    '''
+    creation_count = 0
+    virtual = True
+    '''If set to ``True`` the settings won't be loaded.
+
+    It can be only used as base class for other settings.'''
     name = None
-    '''The unique name used to access this setting in the :class:`Config`
-    container.'''
+    '''The key to access this setting in a :class:`Config` container.'''
     validator = None
-    '''A validating function for this setting. It provided it must be a
-function accepting one positional argument, the value to validate.'''
+    '''A validating function for this setting.
+
+    It provided it must be a function accepting one positional argument,
+    the value to validate.'''
     value = None
     '''The actual value for this setting.'''
     default = None
     '''The default value for this setting.'''
-    def __str__(self):
-        return '{0} ({1})'.format(self.name, self.value)
-    __repr__ = __str__
-
-    def set(self, val, default=False):
-        '''Set *val* as the :attr:`value` for this :class:`SettingBase`.
-If *default* is ``True`` set also the :attr:`default` value.'''
-        if hasattr(self.validator, '__call__'):
-            val = self.validator(val)
-        self.value = val
-        if default:
-            self.default = val
-
-    def get(self):
-        '''Returns :attr:`value`'''
-        return self.value
-
-    def on_start(self):
-        '''Called when pulsar server starts. It can be used to perform
-custom initialization for this :class:`Setting`.'''
-        pass
-    
-    def copy(self, name=None, prefix=None):
-        '''Copy this :class:`SettingBase`'''
-        setting = copy.copy(self)
-        if prefix and not setting.is_global:
-            flags = setting.flags
-            if flags and flags[-1].startswith('--'):
-                # Prefix a setting
-                setting.orig_name = setting.name 
-                setting.name = '%s_%s' % (prefix, setting.name)
-                setting.flags = ['--%s-%s' % (prefix, flags[-1][2:])]
-        if name and not setting.is_global:
-            setting.short = '%s application. %s' % (name , setting.short)
-        return setting
-    
-
-class SimpleSetting(SettingBase):
-
-    def __init__(self, name, value):
-        self.name = name
-        self.value = value
-        self.orig_name = self.name
-
-
-# This works for Python 2 and Python 3
-class Setting(SettingMeta('BaseSettings', (SettingBase,), {'virtual': True})):
-    '''A configuration parameter for pulsar. Parameters can be specified
-on the command line or on a config file.'''
-    creation_count = 0
-    virtual = True
-    '''If set to ``True`` the settings won't be loaded and it can be only used
-as base class for other settings.'''
     nargs = None
-    '''For positional arguments. Same usage as python :mod:`argparse` module.'''
+    '''The number of command-line arguments that should be consumed'''
+    const = None
+    '''A constant value required by some action and nargs selections'''
     app = None
     '''Setting for a specific :class:`Application`.'''
     section = None
@@ -458,13 +399,15 @@ as base class for other settings.'''
     choices = None
     '''Restrict the argument to the choices provided.'''
     type = None
+    '''The type to which the command-line argument should be converted'''
     meta = None
     '''Same usage as ``metavar`` in the python :mod:`argparse` module. It is
     the name for the argument in usage message.'''
     action = None
-    '''Same usage as ``action`` in the python
-    :mod:`argparse.ArgumentParser.add_argument` method.'''
+    '''The basic type of action to be taken when this argument is encountered
+    at the command line'''
     short = None
+    '''Optional shot description string'''
     desc = None
     '''Description string'''
     is_global = False
@@ -478,9 +421,12 @@ as base class for other settings.'''
 
     def __init__(self, name=None, flags=None, action=None, type=None,
                  default=None, nargs=None, desc=None, validator=None,
-                 app=None, meta=None, choices=None):
+                 app=None, meta=None, choices=None, const=None):
+        self.extra = e = {}
         self.app = app or self.app
-        self.choices = choices or self.choices
+        set_if_avail(e, 'choices', choices or self.choices)
+        set_if_avail(e, 'const', const or self.const)
+        set_if_avail(e, 'type', type or self.type, 'string')
         self.default = default if default is not None else self.default
         self.desc = desc or self.desc
         self.flags = flags or self.flags
@@ -488,7 +434,6 @@ as base class for other settings.'''
         self.meta = meta or self.meta
         self.name = name or self.name
         self.nargs = nargs or self.nargs
-        self.type = type or self.type
         self.short = self.short or self.desc
         self.desc = self.desc or self.short
         if self.default is not None:
@@ -504,16 +449,42 @@ as base class for other settings.'''
     def __getstate__(self):
         return self.__dict__.copy()
 
+    def __str__(self):
+        return '{0} ({1})'.format(self.name, self.value)
+    __repr__ = __str__
+
+    def on_start(self):
+        '''Called when pulsar server starts.
+
+        It can be used to perform custom initialization for this
+        :class:`Setting`.
+        '''
+        pass
+
+    def get(self):
+        '''Returns :attr:`value`'''
+        return self.value
+
+    def set(self, val, default=False):
+        '''Set ``val`` as the :attr:`value` for this :class:`Setting`.
+
+        If ``default`` is ``True`` set also the :attr:`default` value.
+        '''
+        if hasattr(self.validator, '__call__'):
+            val = self.validator(val)
+        self.value = val
+        if default:
+            self.default = val
+
     def add_argument(self, parser, set_default=False):
-        '''Add this :class:`Setting` to the *parser*, an instance of
-python :class:`argparse.ArgumentParser`, only if :attr:`flags` or
-:attr:`nargs` and :attr:`name` are defined.'''
+        '''Add this :class:`Setting` to the ``parser``.
+
+        The operation is carried out only if :attr:`flags` or
+        :attr:`nargs` and :attr:`name` are defined.
+        '''
         default = self.default if set_default else None
-        kwargs = {'nargs':self.nargs}
-        if self.type and self.type != 'string':
-            kwargs["type"] = self.type
-        if self.choices:
-            kwargs["choices"] = self.choices
+        kwargs = {'nargs': self.nargs}
+        kwargs.update(self.extra)
         if self.flags:
             args = tuple(self.flags)
             kwargs.update({"dest": self.name,
@@ -521,8 +492,8 @@ python :class:`argparse.ArgumentParser`, only if :attr:`flags` or
                            "default": default,
                            "help": "%s [%s]" % (self.short, self.default)})
             if kwargs["action"] != "store":
-                kwargs.pop("type",None)
-                kwargs.pop("nargs",None)
+                kwargs.pop("type", None)
+                kwargs.pop("nargs", None)
         elif self.nargs and self.name:
             args = (self.name,)
             kwargs.update({'metavar': self.meta or None,
@@ -531,7 +502,21 @@ python :class:`argparse.ArgumentParser`, only if :attr:`flags` or
             # Not added to argparser
             return
         parser.add_argument(*args, **kwargs)
-        
+
+    def copy(self, name=None, prefix=None):
+        '''Copy this :class:`SettingBase`'''
+        setting = copy.copy(self)
+        if prefix and not setting.is_global:
+            flags = setting.flags
+            if flags and flags[-1].startswith('--'):
+                # Prefix a setting
+                setting.orig_name = setting.name
+                setting.name = '%s_%s' % (prefix, setting.name)
+                setting.flags = ['--%s-%s' % (prefix, flags[-1][2:])]
+        if name and not setting.is_global:
+            setting.short = '%s application. %s' % (name, setting.short)
+        return setting
+
 
 def validate_bool(val):
     if isinstance(val, bool):
@@ -546,6 +531,7 @@ def validate_bool(val):
     else:
         raise ValueError("Invalid boolean: %s" % val)
 
+
 def validate_pos_int(val):
     if not isinstance(val, int):
         val = int(val, 0)
@@ -556,12 +542,14 @@ def validate_pos_int(val):
         raise ValueError("Value must be positive: %s" % val)
     return val
 
+
 def validate_pos_float(val):
     val = float(val)
     if val < 0:
         raise ValueError("Value must be positive: %s" % val)
     return val
 
+
 def validate_string(val):
     va = native_str(val)
     if va is None:
@@ -570,19 +558,22 @@ def validate_string(val):
         raise TypeError("Not a string: %s" % val)
     return va.strip()
 
+
 def validate_list(val):
     if val and not isinstance(val, (list, tuple)):
         raise TypeError("Not a list: %s" % val)
     return list(val)
 
+
 def validate_dict(val):
     if val and not isinstance(val, dict):
         raise TypeError("Not a dictionary: %s" % val)
     return val
 
+
 def validate_callable(arity):
     def _validate_callable(val):
-        if not hasattr(val,'__call__'):
+        if not hasattr(val, '__call__'):
             raise TypeError("Value is not callable: %s" % val)
         if not inspect.isfunction(val):
             cval = val.__call__
@@ -590,31 +581,40 @@ def validate_callable(arity):
         else:
             discount = 0
             cval = val
-        if arity != len(inspect.getargspec(cval)[0]) - discount:
+        result = inspect.getargspec(cval)
+        nargs = len(result.args) - discount
+        if result.defaults:
+            group = tuple(range(nargs-len(result.defaults), nargs+1))
+        else:
+            group = (nargs,)
+        if arity not in group:
             raise TypeError("Value must have an arity of: %s" % arity)
         return val
     return _validate_callable
 
 
-def make_optparse_options(apps=None, exclude=None, include=None): # pragma nocover
-    '''Create a tuple of optparse options'''
+def make_optparse_options(apps=None, exclude=None, include=None):
+    '''Create a tuple of optparse options.'''
     from optparse import make_option
+
     class AddOptParser(list):
         def add_argument(self, *args, **kwargs):
             self.append(make_option(*args, **kwargs))
+
     config = Config(apps=apps, exclude=None, include=None)
     parser = AddOptParser()
     config.add_to_parser(parser)
     return tuple(parser)
-    
 
-################################################################################
+
+############################################################################
 ##    Global Server Settings
-
 section_docs['Global Server Settings'] = '''
 These settings are global in the sense that they are used by the arbiter
 as well as all pulsar workers. They are server configuration parameters.
 '''
+
+
 class Global(Setting):
     virtual = True
     section = "Global Server Settings"
@@ -632,7 +632,7 @@ class ConfigFile(Global):
         paramaters can be specified.
         """
 
-    
+
 class HttpProxyServer(Global):
     name = "http_proxy"
     flags = ["--http-proxy"]
@@ -640,8 +640,9 @@ class HttpProxyServer(Global):
     desc = """\
         The HTTP proxy server to use with HttpClient.
         """
+
     def on_start(self):
-        if self.value:
+        if self.value:  # pragma    nocover
             os.environ['http_proxy'] = self.value
             os.environ['https_proxy'] = self.value
 
@@ -656,8 +657,8 @@ class HttpParser(Global):
         """
 
     def on_start(self):
-        if self.value:
-            from pulsar.utils.httpurl import setDefaultHttpParser 
+        if self.value:  # pragma    nocover
+            from pulsar.utils.httpurl import setDefaultHttpParser
             setDefaultHttpParser(PyHttpParser)
 
 
@@ -747,9 +748,9 @@ class Loglevel(Global):
     validator = validate_string
     default = "info"
     desc = """The granularity of log outputs.
-            
+
             Valid level names are:
-            
+
              * debug
              * info
              * warning
@@ -757,6 +758,7 @@ class Loglevel(Global):
              * critical
              """
 
+
 class LogHandlers(Global):
     name = "loghandlers"
     flags = ["--log-handlers"]
@@ -787,8 +789,8 @@ class Procname(Global):
 
         This affects things like ``ps`` and ``top``. If you're going to be
         running more than one instance of Pulsar you'll probably want to set a
-        name to tell them apart. This requires that you install the setproctitle
-        module.
+        name to tell them apart. This requires that you install the
+        setproctitle module.
 
         It defaults to 'pulsar'.
         """
@@ -802,15 +804,27 @@ class DefaultProcName(Global):
         Internal setting that is adjusted for each type of application.
         """
 
-################################################################################
-##    Worker Processes
 
+class Coverage(Global):
+    name = "coverage"
+    flags = ["--coverage"]
+    validator = validate_bool
+    action = "store_true"
+    default = False
+    desc = """\
+        Collect code coverage from all spawn actors.
+        """
+
+
+############################################################################
+##    Worker Processes
 section_docs['Worker Processes'] = '''
 This group of configuration parameters control the number of actors
 for a given :class:`pulsar.Monitor`, the type of concurreny of the server and
 other actor-specific parameters.
 '''
-    
+
+
 class Workers(Setting):
     name = "workers"
     section = "Worker Processes"
@@ -820,14 +834,15 @@ class Workers(Setting):
     default = 1
     desc = """\
         The number of workers for handling requests.
-        
-If using a multi-process concurrency, a number in the
-the ``2-4 x NUM_CORES`` range should be good. If you are using threads this
-number can be higher."""
+
+        If using a multi-process concurrency, a number in the
+        the ``2-4 x NUM_CORES`` range should be good. If you are using
+        threads this number can be higher.
+        """
 
 
 class Concurrency(Setting):
-    inherit = True # Inherited by the arbiter if an application is first created
+    inherit = True  # Inherited by the arbiter
     name = "concurrency"
     section = "Worker Processes"
     flags = ["--concurrency"]
@@ -855,6 +870,7 @@ class MaxRequests(Setting):
         restarts are disabled.
         """
 
+
 class Timeout(Setting):
     name = "timeout"
     section = "Worker Processes"
@@ -876,15 +892,15 @@ class ThreadWorkers(Setting):
     default = 1
     desc = """\
         The number of threads in an actor thread pool.
-        
-        The thread pool is used by actors to perform CPU intensive calculations.
-        In this way the actor main thread is free to listen to events on file
-        descriptors and process them as quick as possible."""
+
+        The thread pool is used by actors to perform CPU intensive
+        calculations. In this way the actor main thread is free to listen
+        to events on file descriptors and process them as quick as possible.
+        """
 
 
-################################################################################
+############################################################################
 ##    APPLICATION HOOKS
-
 section_docs['Application Hooks'] = '''
 Application hooks are functions which can be specified in a
 :ref:`config <setting-config>` file to perform custom tasks in a pulsar server.
@@ -893,7 +909,8 @@ the various components of a pulsar application.
 
 All application hooks are functions which accept one parameter only, the actor
 invoking the function.'''
-    
+
+
 class Postfork(Setting):
     name = "post_fork"
     section = "Application Hooks"
@@ -905,7 +922,7 @@ class Postfork(Setting):
 
         The event loop is not yet available.
         """
-        
+
 
 class WhenReady(Setting):
     name = "when_ready"
@@ -915,12 +932,12 @@ class WhenReady(Setting):
     default = staticmethod(pass_through)
     desc = """\
         Called just before a worker starts its event loop.
-        
+
         This is a chance to setup :class:`pulsar.EventLoop` callbacks which
         can run periodically, at every loop or when some defined events occur.
         """
-        
-        
+
+
 class WhenExit(Setting):
     name = "when_exit"
     section = "Application Hooks"
@@ -929,7 +946,7 @@ class WhenExit(Setting):
     default = staticmethod(pass_through)
     desc = """\
         Called just before an actor is garbadge collected.
-        
+
         This is a chance to check the actor status if needed.
         """
 
@@ -943,7 +960,7 @@ class ConnectionMade(Setting):
     desc = """\
         Called after a new connection is made.
 
-        The callable needs to accept one instance variables for the
+        The callable needs to accept one parameter for the
         connection instance.
         """
 
@@ -957,7 +974,7 @@ class ConnectionLost(Setting):
     desc = """
         Called after a connection is lost.
 
-        The callable needs to accept one instance variables for the
+        The callable needs to accept one parameter for the
         connection instance.
         """
 
@@ -969,10 +986,10 @@ class PreRequest(Setting):
     type = "callable"
     default = staticmethod(pass_through)
     desc = """\
-        Called just before a worker processes the request.
+        Called just before an application server processes a request.
 
-        The callable needs to accept two instance variables for the Worker and
-        the Request.
+        The callable needs to accept one parameters for the
+        consumer.
         """
 
 
@@ -983,8 +1000,8 @@ class PostRequest(Setting):
     type = "callable"
     default = staticmethod(pass_through)
     desc = """\
-        Called after a worker processes the request.
+        Called after an application server processes a request.
 
-        The callable needs to accept two instance variables for the Worker and
-        the Request.
+        The callable needs to accept one parameter for the
+        consumer.
         """
