@@ -20,8 +20,8 @@ HTTP Headers
 .. autoclass:: Headers
    :members:
    :member-order: bysource
-   
-   
+
+
 .. _tools-http-parser:
 
 HTTP Parser
@@ -45,106 +45,77 @@ parse cookie
 import os
 import sys
 import re
-import io
 import string
 import time
 import mimetypes
 import platform
-import codecs
 import socket
 import logging
-from functools import reduce
 from hashlib import sha1, md5
 from uuid import uuid4
-from datetime import datetime, timedelta
 from email.utils import formatdate
 from io import BytesIO
 import zlib
 from collections import deque
-from copy import copy
+
+from .structures import mapping_iterator, OrderedDict
+from .pep import ispy3k, iteritems, itervalues, to_bytes, native_str
 
 try:
     from http_parser.parser import HttpParser as CHttpParser
     hasextensions = True
     _Http_Parser = CHttpParser
-except ImportError: #pragma    nocover
+except ImportError:  # pragma    nocover
     hasextensions = False
     _Http_Parser = None
 
-def setDefaultHttpParser(parser):
+try:
+    from select import poll, POLLIN
+except ImportError:   # pragma    nocover
+    poll = False
+    try:
+        from select import select
+    except ImportError:  # pragma    nocover
+        select = False
+
+
+def setDefaultHttpParser(parser):   # pragma    nocover
     global _Http_Parser
     _Http_Parser = parser
-    
+
+
 def http_parser(**kwargs):
     global _Http_Parser
     return _Http_Parser(**kwargs)
 
-from .structures import mapping_iterator, OrderedDict
-from .pep import ispy3k, ispy26, iteritems, itervalues
 
 create_connection = socket.create_connection
 LOGGER = logging.getLogger('httpurl')
 
-try:
-    from select import poll, POLLIN
-except ImportError: #pragma    nocover
-    poll = False
-    try:
-        from select import select
-    except ImportError: #pragma    nocover
-        select = False
-        
 try:    # Compiled with SSL?
     BaseSSLError = None
     ssl = None
     import ssl
     BaseSSLError = ssl.SSLError
-except (ImportError, AttributeError):   # pragma : no cover 
+except (ImportError, AttributeError):   # pragma : no cover
     pass
 
-if ispy3k: # Python 3
+if ispy3k:  # Python 3
     from urllib import request as urllibr
     from http import client as httpclient
-    from urllib.parse import quote, unquote, urlencode, urlparse, urlsplit,\
-                             parse_qs, parse_qsl, splitport, urlunparse,\
-                             urljoin
+    from urllib.parse import (quote, unquote, urlencode, urlparse, urlsplit,
+                              parse_qs, parse_qsl, splitport, urlunparse,
+                              urljoin)
     from http.client import responses
     from http.cookiejar import CookieJar, Cookie
-    from http.cookies import SimpleCookie, BaseCookie, Morsel, CookieError
+    from http.cookies import SimpleCookie, BaseCookie, CookieError
 
     string_type = str
     getproxies_environment = urllibr.getproxies_environment
     ascii_letters = string.ascii_letters
-    zip = zip
-    map = map
-    range = range
     chr = chr
     is_string = lambda s: isinstance(s, str)
 
-    def to_bytes(s, encoding=None, errors=None):
-        errors = errors or 'strict'
-        encoding = encoding or 'utf-8'
-        if isinstance(s, bytes):
-            if encoding != 'utf-8':
-                return s.decode('utf-8', errors).encode(encoding, errors)
-            else:
-                return s
-        else:
-            return ('%s'%s).encode(encoding, errors)
-
-    def to_string(s, encoding=None, errors='strict'):
-        """Inverse of to_bytes"""
-        if isinstance(s, bytes):
-            return s.decode(encoding or 'utf-8', errors)
-        else:
-            return str(s)
-
-    def native_str(s, encoding=None):
-        if isinstance(s, bytes):
-            return s.decode(encoding or 'utf-8')
-        else:
-            return s
-
     def force_native_str(s, encoding=None):
         if isinstance(s, bytes):
             return s.decode(encoding or 'utf-8')
@@ -156,18 +127,16 @@ if ispy3k: # Python 3
 else:   # pragma : no cover
     import urllib2 as urllibr
     import httplib as httpclient
-    from urllib import quote, unquote, urlencode, getproxies_environment,\
-                        splitport
-    from urlparse import urlparse, urlsplit, parse_qs, urlunparse, urljoin,\
-                         parse_qsl
+    from urllib import (quote, unquote, urlencode, getproxies_environment,
+                        splitport)
+    from urlparse import (urlparse, urlsplit, parse_qs, urlunparse, urljoin,
+                          parse_qsl)
     from httplib import responses
     from cookielib import CookieJar, Cookie
-    from Cookie import SimpleCookie, BaseCookie, Morsel, CookieError
-    from itertools import izip as zip, imap as map
+    from Cookie import SimpleCookie, BaseCookie, CookieError
 
     string_type = unicode
     ascii_letters = string.letters
-    range = xrange
     chr = unichr
     is_string = lambda s: isinstance(s, unicode)
 
@@ -196,30 +165,7 @@ else:   # pragma : no cover
             if err is not None:
                 raise err
             else:
-                raise error("getaddrinfo returns an empty list")
-    
-    def to_bytes(s, encoding=None, errors='strict'):
-        encoding = encoding or 'utf-8'
-        if isinstance(s, bytes):
-            if encoding != 'utf-8':
-                return s.decode('utf-8', errors).encode(encoding, errors)
-            else:
-                return s
-        else:
-            return unicode(s).encode(encoding, errors)
-
-    def to_string(s, encoding=None, errors='strict'):
-        """Inverse of to_bytes"""
-        if isinstance(s, bytes):
-            return s.decode(encoding or 'utf-8', errors)
-        else:
-            return unicode(s)
-
-    def native_str(s, encoding=None):
-        if isinstance(s, unicode):
-            return s.encode(encoding or 'utf-8')
-        else:
-            return s
+                raise Exception("getaddrinfo returns an empty list")
 
     def force_native_str(s, encoding=None):
         if isinstance(s, unicode):
@@ -234,6 +180,7 @@ URLError = urllibr.URLError
 request_host = urllibr.request_host
 parse_http_list = urllibr.parse_http_list
 
+
 class SSLError(HTTPError):
     "Raised when SSL certificate fails in an HTTPS connection."
     pass
@@ -241,9 +188,9 @@ class SSLError(HTTPError):
 ####################################################    URI & IRI SUFF
 #
 # The reserved URI characters (RFC 3986 - section 2.2)
-#Default is charset is "iso-8859-1" (latin-1) from section 3.7.1
-#http://www.ietf.org/rfc/rfc2616.txt
-DEFAULT_CHARSET = 'iso-8859-1'
+# Default is charset is "iso-8859-1" (latin-1) from section 3.7.1
+# http://www.ietf.org/rfc/rfc2616.txt
+DEFAULT_CHARSET = 'ISO-8859-1'
 URI_GEN_DELIMS = frozenset(':/?#[]@')
 URI_SUB_DELIMS = frozenset("!$&'()*+,;=")
 URI_RESERVED_SET = URI_GEN_DELIMS.union(URI_SUB_DELIMS)
@@ -257,6 +204,7 @@ HEADER_TOKEN_CHARS = frozenset("!#$%&'*+-.0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
 escape = lambda s: quote(s, safe='~')
 urlquote = lambda iri: quote(iri, safe=URI_RESERVED_CHARS)
 
+
 def _gen_unquote(uri):
     unreserved_set = URI_UNRESERVED_SET
     for n, part in enumerate(force_native_str(uri).split('%')):
@@ -272,12 +220,14 @@ def _gen_unquote(uri):
                     yield '%' + part
             else:
                 yield '%' + part
-                
+
+
 def unquote_unreserved(uri):
     """Un-escape any percent-escape sequences in a URI that are unreserved
 characters. This leaves all reserved, illegal and non-ASCII bytes encoded."""
     return ''.join(_gen_unquote(uri))
 
+
 def requote_uri(uri):
     """Re-quote the given URI.
 
@@ -289,6 +239,7 @@ def requote_uri(uri):
     # or '%')
     return quote(unquote_unreserved(uri), safe=URI_SAFE_CHARS)
 
+
 def iri_to_uri(iri, kwargs=None):
     '''Convert an Internationalized Resource Identifier (IRI) portion to a URI
 portion that is suitable for inclusion in a URL.
@@ -298,25 +249,38 @@ Returns an ASCII native string containing the encoded result.'''
         return iri
     iri = force_native_str(iri)
     if kwargs:
-        iri = '%s?%s'%(iri,'&'.join(('%s=%s' % kv for kv in iteritems(kwargs))))
+        iri = '%s?%s' % (iri, '&'.join(('%s=%s' % kv for kv in
+                                        iteritems(kwargs))))
     return urlquote(unquote_unreserved(iri))
 
+
 def host_and_port(host):
     host, port = splitport(host)
     return host, int(port) if port else None
 
+
 def default_port(scheme):
-    if scheme == "http":
+    if scheme in ("http", "ws"):
         return '80'
-    elif scheme == "https":
+    elif scheme in ("https", "wss"):
         return '443'
-    
+
+
 def host_and_port_default(scheme, host):
     host, port = splitport(host)
     if not port:
         port = default_port(scheme)
     return host, port
 
+
+def host_no_default_port(scheme, netloc):
+    host, port = splitport(netloc)
+    if port and port == default_port(scheme):
+        return host
+    else:
+        return netloc
+
+
 def get_hostport(scheme, full_host):
     host, port = host_and_port(full_host)
     if port is None:
@@ -326,56 +290,36 @@ def get_hostport(scheme, full_host):
             try:
                 port = int(host[i+1:])
             except ValueError:
-                if host[i+1:] == "": # http://foo.com:/ == http://foo.com/
+                if host[i+1:] == "":  # http://foo.com:/ == http://foo.com/
                     port = default_port(scheme)
                 else:
                     raise httpclient.InvalidURL("nonnumeric port: '%s'"
-                                                 % host[i+1:])
+                                                % host[i+1:])
             host = host[:i]
         else:
             port = default_port(scheme)
         if host and host[0] == '[' and host[-1] == ']':
             host = host[1:-1]
-    return host, port
+    return host, int(port)
+
 
 def remove_double_slash(route):
     if '//' in route:
         route = re.sub('/+', '/', route)
     return route
 
-def is_closed(sock):    #pragma nocover
-    """Check if socket is connected."""
-    if not sock:
-        return False
-    try:
-        if not poll:
-            if not select:
-                return False
-            try:
-                return bool(select([sock], [], [], 0.0)[0])
-            except socket.error:
-                return True
-        # This version is better on platforms that support it.
-        p = poll()
-        p.register(sock, POLLIN)
-        for (fno, ev) in p.poll(0.0):
-            if fno == sock.fileno():
-                # Either data is buffered (bad), or the connection is dropped.
-                return True
-    except Exception:
-        return True
 
 ####################################################    CONTENT TYPES
 JSON_CONTENT_TYPES = ('application/json',
                       'application/javascript',
                       'text/json',
                       'text/x-json')
-
 ####################################################    REQUEST METHODS
 ENCODE_URL_METHODS = frozenset(['DELETE', 'GET', 'HEAD', 'OPTIONS'])
 ENCODE_BODY_METHODS = frozenset(['PATCH', 'POST', 'PUT', 'TRACE'])
 REDIRECT_CODES = (301, 302, 303, 307)
 
+
 def has_empty_content(status, method=None):
     '''204, 304 and 1xx codes have no content'''
     if status == httpclient.NO_CONTENT or\
@@ -386,20 +330,23 @@ def has_empty_content(status, method=None):
     else:
         return False
 
+
 def is_succesful(status):
     '''2xx status is succesful'''
     return status >= 200 and status < 300
 
+
 ####################################################    HTTP HEADERS
 WEBSOCKET_VERSION = (8, 13)
 HEADER_FIELDS = {'general': frozenset(('Cache-Control', 'Connection', 'Date',
-                                       'Pragma', 'Trailer','Transfer-Encoding',
+                                       'Pragma', 'Trailer',
+                                       'Transfer-Encoding',
                                        'Upgrade', 'Sec-WebSocket-Extensions',
                                        'Sec-WebSocket-Protocol',
-                                       'Via','Warning')),
-                 # The request-header fields allow the client to pass additional
-                 # information about the request, and about the client itself,
-                 # to the server.
+                                       'Via', 'Warning')),
+                 # The request-header fields allow the client to pass
+                 # additional information about the request, and about the
+                 # client to the server.
                  'request': frozenset(('Accept', 'Accept-Charset',
                                        'Accept-Encoding', 'Accept-Language',
                                        'Authorization',
@@ -436,19 +383,20 @@ HEADER_FIELDS = {'general': frozenset(('Cache-Control', 'Connection', 'Date',
                                       'Content-Range', 'Content-Type',
                                       'Expires', 'Last-Modified'))}
 
-CLIENT_HEADER_FIELDS = HEADER_FIELDS['general'].union(HEADER_FIELDS['entity'],
-                                                      HEADER_FIELDS['request'])
-SERVER_HEADER_FIELDS = HEADER_FIELDS['general'].union(HEADER_FIELDS['entity'],
-                                                      HEADER_FIELDS['response'])
+CLIENT_HEADER_FIELDS = HEADER_FIELDS['general'].union(
+    HEADER_FIELDS['entity'], HEADER_FIELDS['request'])
+SERVER_HEADER_FIELDS = HEADER_FIELDS['general'].union(
+    HEADER_FIELDS['entity'], HEADER_FIELDS['response'])
 ALL_HEADER_FIELDS = CLIENT_HEADER_FIELDS.union(SERVER_HEADER_FIELDS)
-ALL_HEADER_FIELDS_DICT = dict(((k.lower(),k) for k in ALL_HEADER_FIELDS))
+ALL_HEADER_FIELDS_DICT = dict(((k.lower(), k) for k in ALL_HEADER_FIELDS))
 
 TYPE_HEADER_FIELDS = {'client': CLIENT_HEADER_FIELDS,
                       'server': SERVER_HEADER_FIELDS,
                       'both': ALL_HEADER_FIELDS}
 
 header_type = {0: 'client', 1: 'server', 2: 'both'}
-header_type_to_int = dict(((v,k) for k,v in header_type.items()))
+header_type_to_int = dict(((v, k) for k, v in header_type.items()))
+
 
 def capfirst(x):
     x = x.strip()
@@ -456,10 +404,12 @@ def capfirst(x):
         return x[0].upper() + x[1:].lower()
     else:
         return x
-    
+
+
 def capheader(name):
     return '-'.join((b for b in (capfirst(n) for n in name.split('-')) if b))
 
+
 def header_field(name, HEADERS_SET=None, strict=False):
     name = name.lower()
     if name.startswith('x-'):
@@ -473,6 +423,7 @@ def header_field(name, HEADERS_SET=None, strict=False):
         elif not strict:
             return capheader(name)
 
+
 def quote_header_value(value, extra_chars='', allow_token=True):
     """Quote a header value if necessary.
 
@@ -487,6 +438,7 @@ def quote_header_value(value, extra_chars='', allow_token=True):
             return value
     return '"%s"' % value.replace('\\', '\\\\').replace('"', '\\"')
 
+
 def unquote_header_value(value, is_filename=False):
     """Unquotes a header value. Reversal of :func:`quote_header_value`.
 This does not use the real unquoting but what browsers are actually
@@ -507,6 +459,7 @@ using for quoting.
             return value.replace('\\\\', '\\').replace('\\"', '"')
     return value
 
+
 def parse_dict_header(value):
     """Parse lists of key, value pairs as described by RFC 2068 Section 2 and
     convert them into a python dict:
@@ -542,13 +495,14 @@ def parse_dict_header(value):
 
 class Headers(object):
     '''Utility for managing HTTP headers for both clients and servers.
+
 It has a dictionary like interface with few extra functions to facilitate
-the insertion of multiple header values. Header fields are **case insensitive**,
-therefore doing::
+the insertion of multiple header values. Header fields are
+**case insensitive**, therefore doing::
 
     >>> h = Headers()
     >>> h['Content-Length'] = '1050'
-    
+
 is equivalent to
 
     >>> h['content-length'] = '1050'
@@ -562,13 +516,13 @@ http://www.w3.org/Protocols/rfc2616/rfc2616.html:
 
 .. epigraph::
 
-    The order in which header fields with differing field names are received is not
-    significant. However, it is "good practice" to send general-header fields first,
-    followed by request-header or response-header fields, and ending with
-    the entity-header fields.
-    
+    The order in which header fields with differing field names are received
+    is not significant. However, it is "good practice" to send general-header
+    fields first, followed by request-header or response-header fields, and
+    ending with the entity-header fields.
+
     -- rfc2616 section 4.2
-    
+
 The strict parameter is rarely used and it forces the omission on non-standard
 header fields.
 '''
@@ -594,7 +548,7 @@ header fields.
         return '\r\n'.join(self._ordered())
 
     def __bytes__(self):
-        return str(self).encode('latin-1')
+        return str(self).encode(DEFAULT_CHARSET)
 
     def __iter__(self):
         for k, values in iteritems(self._headers):
@@ -608,10 +562,29 @@ header fields.
         return header_type_to_int.get(self.kind)
 
     def update(self, iterable):
-        """Extend the headers with a dictionary or an iterable yielding keys
-and values."""
+        """Extend the headers.
+
+        :param iterable: a dictionary or an iterable over keys, vaues tuples.
+        """
         for key, value in mapping_iterator(iterable):
-            self[key] = value
+            self.add_header(key, value)
+
+    def override(self, iterable):
+        '''Extend headers by overriding fields form iterable.
+
+        :param iterable: a dictionary or an iterable over keys, vaues tuples.
+        '''
+        seen = set()
+        for key, value in mapping_iterator(iterable):
+            key = key.lower()
+            if key in seen:
+                self.add_header(key, value)
+            else:
+                seen.add(key)
+                self[key] = value
+
+    def copy(self):
+        return self.__class__(self, kind=self.kind, strict=self.strict)
 
     def __contains__(self, key):
         return header_field(key) in self._headers
@@ -630,19 +603,19 @@ and values."""
             self._headers[key] = value
 
     def get(self, key, default=None):
-        '''Get the value at header field ``key`` as a comma separated
-string of values. For example::
+        '''Get the field value at ``key`` as comma separated values.
+        For example::
 
-    >>> from pulsar.utils.httpurl import Headers
-    >>> h = Headers(kind='client')
-    >>> h.add_header('accept-encoding', 'gzip')
-    >>> h.add_header('accept-encoding', 'deflate')
-    >>> h.get('accept-encoding')
-    
-results in::
+            >>> from pulsar.utils.httpurl import Headers
+            >>> h = Headers(kind='client')
+            >>> h.add_header('accept-encoding', 'gzip')
+            >>> h.add_header('accept-encoding', 'deflate')
+            >>> h.get('accept-encoding')
 
-    'gzip, deflate'
-'''
+        results in::
+
+            'gzip, deflate'
+        '''
         if key in self:
             return self.__getitem__(key)
         else:
@@ -658,7 +631,7 @@ For example::
     >>> h.add_header('accept-encoding', 'gzip')
     >>> h.add_header('accept-encoding', 'deflate')
     >>> h.get_all('accept-encoding')
-    
+
 results in::
 
     ['gzip', 'deflate']
@@ -672,42 +645,46 @@ results in::
             if c.lower() == value:
                 return True
         return False
-    
+
     def pop(self, key, *args):
         return self._headers.pop(header_field(key), *args)
 
-    def copy(self):
-        return self.__class__(self, kind=self.kind)
-    
     def clear(self):
-        '''Same as :meth:`dict.clear`, it removes all headers.'''
+        '''Same as :meth:`dict.clear`, it removes all headers.
+        '''
         self._headers.clear()
 
-    def getheaders(self, key):
-        '''Required by cookielib. If the key is not available,
-it returns an empty list.'''
+    def getheaders(self, key):  # pragma    nocover
+        '''Required by cookielib in python 2.
+
+        If the key is not available, it returns an empty list.
+        '''
         return self._headers.get(header_field(key), [])
 
-    def headers(self):
-        return list(self)
-
     def add_header(self, key, value, **params):
-        '''Add *value* to *key* header. If the header is already available,
-append the value to the list.'''
+        '''Add ``value`` to ``key`` header.
+
+        If the header is already available, append the value to the list.
+        '''
         key = header_field(key, self.all_headers, self.strict)
         if key and value:
             values = self._headers.get(key, [])
             lower_values = [v.lower() for v in values]
             for value in self.get_values(value):
-                lower_value = value.lower() 
+                if params:
+                    value = '%s; %s' % (value, '; '.join(('%s=%s' % kv for kv
+                                                          in params.items())))
+                lower_value = value.lower()
                 if lower_value not in lower_values:
                     lower_values.append(lower_value)
                     values.append(value)
             self._headers[key] = values
-                
+
     def remove_header(self, key, value=None):
-        '''Remove the header at ``key``, If ``value`` is provided, it removes
-only that value if found.'''
+        '''Remove the header at ``key``.
+
+        If ``value`` is provided, it removes only that value if found.
+        '''
         key = header_field(key, self.all_headers, self.strict)
         if key:
             if value:
@@ -724,13 +701,14 @@ only that value if found.'''
                 return self._headers.pop(key, None)
 
     def flat(self, version, status):
-    	'''Full headers bytes representation'''
-    	vs = version + (status, self)
-    	return ('HTTP/%s.%s %s\r\n%s' % vs).encode(DEFAULT_CHARSET)
+        '''Full headers bytes representation'''
+        vs = version + (status, self)
+        return ('HTTP/%s.%s %s\r\n%s' % vs).encode(DEFAULT_CHARSET)
 
     def _ordered(self):
         hf = HEADER_FIELDS
-        order = (('general',[]), ('request',[]), ('response',[]), ('entity',[]))
+        order = (('general', []), ('request', []),
+                 ('response', []), ('entity', []))
         headers = self._headers
         for key in headers:
             for name, group in order:
@@ -744,7 +722,7 @@ only that value if found.'''
                 yield "%s: %s" % (k, ', '.join(headers[k]))
         yield ''
         yield ''
-        
+
     ##    INTERNALS
     def get_values(self, value):
         return [v for v in (v.strip() for v in value.split(',')) if v]
@@ -763,6 +741,7 @@ BAD_FIRST_LINE = 0
 INVALID_HEADER = 1
 INVALID_CHUNK = 2
 
+
 class InvalidRequestLine(Exception):
     """ error raised when first line is invalid """
 
@@ -818,7 +797,7 @@ OTHER DEALINGS IN THE SOFTWARE.'''
         self._path = None
         self._query_string = None
         self._kind = kind
-        self._fragment= None
+        self._fragment = None
         self._headers = OrderedDict()
         self._chunked = False
         self._body = []
@@ -891,7 +870,9 @@ OTHER DEALINGS IN THE SOFTWARE.'''
     def execute(self, data, length):
         # end of body can be passed manually by putting a length of 0
         if length == 0:
-            return self.close(length)
+            self.__on_message_complete = True
+            return length
+        #
         data = bytes(data)
         # start to parse
         nb_parsed = 0
@@ -904,7 +885,8 @@ OTHER DEALINGS IN THE SOFTWARE.'''
                 else:
                     self.__on_firstline = True
                     self._buf.append(data[:idx])
-                    first_line = native_str(b''.join(self._buf),DEFAULT_CHARSET)
+                    first_line = native_str(b''.join(self._buf),
+                                            DEFAULT_CHARSET)
                     rest = data[idx+2:]
                     data = b''
                     if self._parse_firstline(first_line):
@@ -920,9 +902,6 @@ OTHER DEALINGS IN THE SOFTWARE.'''
                     to_parse = b''.join(self._buf)
                     ret = self._parse_headers(to_parse)
                     if ret is False:
-                        if to_parse == b'\r\n':
-                            self._buf = []
-                            return self.close(length)
                         return length
                     nb_parsed = nb_parsed + (len(to_parse) - ret)
                 except InvalidHeader as e:
@@ -947,18 +926,9 @@ OTHER DEALINGS IN THE SOFTWARE.'''
             else:
                 return 0
 
-    def close(self, length):
-        self.__on_message_begin = True
-        self.__on_message_complete = True
-        if not self._buf and self.__on_firstline:
-            self.__on_headers_complete = True
-            return length
-        else:
-            return length+len(self._buf)
-        
     def _parse_firstline(self, line):
         try:
-            if self.kind == 2: # auto detect
+            if self.kind == 2:  # auto detect
                 try:
                     self._parse_request_line(line)
                 except InvalidRequestLine:
@@ -1014,8 +984,12 @@ OTHER DEALINGS IN THE SOFTWARE.'''
         self._version = (int(match.group(1)), int(match.group(2)))
 
     def _parse_headers(self, data):
+        if data == b'\r\n':
+            self.__on_headers_complete = True
+            self._buf = []
+            return 0
         idx = data.find(b'\r\n\r\n')
-        if idx < 0: # we don't have all headers
+        if idx < 0:  # we don't have all headers
             return False
         chunk = native_str(data[:idx], DEFAULT_CHARSET)
         # Split lines on \r\n keeping the \r\n on each line
@@ -1045,7 +1019,14 @@ OTHER DEALINGS IN THE SOFTWARE.'''
         clen = self._headers.get('content-length')
         te = self._headers.get('transfer-encoding', '').lower()
         self._chunked = (te == 'chunked')
-        if clen and not self._chunked:
+        #
+        status = self._status_code
+        if status and (status == httpclient.NO_CONTENT or
+                       status == httpclient.NOT_MODIFIED or
+                       100 <= status < 200 or      # 1xx codes
+                       self._method == "HEAD"):
+            clen = 0
+        elif clen is not None:
             try:
                 clen = int(clen)
             except ValueError:
@@ -1053,15 +1034,12 @@ OTHER DEALINGS IN THE SOFTWARE.'''
             else:
                 if clen < 0:  # ignore nonsensical negative lengths
                     clen = None
+        #
+        if clen is None:
+            self._clen_rest = sys.maxsize
         else:
-            clen = None
-        status = self._status_code
-        if status and (status == httpclient.NO_CONTENT or
-            status == httpclient.NOT_MODIFIED or
-            100 <= status < 200 or      # 1xx codes
-            self._method == "HEAD"):
-            clen = 0
-        self._clen_rest = self._clen = clen
+            self._clen_rest = self._clen = clen
+        #
         # detect encoding and set decompress object
         if self.decompress:
             encoding = self._headers.get('content-encoding')
@@ -1072,22 +1050,30 @@ OTHER DEALINGS IN THE SOFTWARE.'''
         rest = data[idx+4:]
         self._buf = [rest]
         self.__on_headers_complete = True
+        self.__on_message_begin = True
         return len(rest)
 
     def _parse_body(self):
         data = b''.join(self._buf)
+        #
         if not self._chunked:
-            if self._clen_rest is not None:
-                self._clen_rest -= len(data)
-            # maybe decompress
-            if self.__decompress_obj is not None:
-                data = self.__decompress_obj.decompress(data)
-            self._partial_body = True
-            if data:
-                self._body.append(data)
-            self._buf = []
-            if self._clen_rest is None or self._clen_rest <= 0:
-                self.__on_message_complete = True
+            #
+            if not data and self._clen is None:
+                if not self._status:    # message complete only for servers
+                    self.__on_message_complete = True
+            else:
+                if self._clen_rest is not None:
+                    self._clen_rest -= len(data)
+                # maybe decompress
+                if self.__decompress_obj is not None:
+                    data = self.__decompress_obj.decompress(data)
+                self._partial_body = True
+                if data:
+                    self._body.append(data)
+                self._buf = []
+                if self._clen_rest <= 0:
+                    self.__on_message_complete = True
+            return
         else:
             try:
                 size, rest = self._parse_chunk_size(data)
@@ -1109,8 +1095,9 @@ OTHER DEALINGS IN THE SOFTWARE.'''
                 body_part = self.__decompress_obj.decompress(body_part)
             self._partial_body = True
             self._body.append(body_part)
-            self._buf = [rest[2:]]
-            return len(rest)
+            rest = rest[2:]
+            self._buf = [rest] if rest else []
+            return len(rest) + 2
 
     def _parse_chunk_size(self, data):
         idx = data.find(b'\r\n')
@@ -1132,47 +1119,10 @@ OTHER DEALINGS IN THE SOFTWARE.'''
         if data[:2] == b'\r\n':
             self._trailers = self._parse_headers(data[:idx])
 
-if not hasextensions:
+if not hasextensions:   # pragma    nocover
     setDefaultHttpParser(HttpParser)
-    
-################################################################################
-##    HTTP CLIENT
-################################################################################
 
-class WWWAuthenticate(object):
-    _require_quoting = frozenset(['domain', 'nonce', 'opaque', 'realm'])
-    
-    def __init__(self, type, **options):
-        self.type = type
-        self.options = options
-    
-    @classmethod
-    def basic(cls, realm='authentication required'):
-        """Clear the auth info and enable basic auth."""
-        return cls('basic', realm=realm)
-    
-    @classmethod    
-    def digest(cls, realm, nonce, qop=('auth',), opaque=None,
-               algorithm=None, stale=None):
-        options = {}
-        if stale:
-            options['stale'] = 'TRUE'
-        if opaque is not None:
-            options['opaque'] = opaque
-        if algorithm is not None:
-            options['algorithm'] = algorithm
-        return cls('digest', realm=realm, nonce=nonce,
-                   qop=', '.join(qop), **options)
-    
-    def __str__(self):
-        """Convert the stored values into a WWW-Authenticate header."""
-        return '%s %s' % (self.type.title(), ', '.join((
-            '%s=%s' % (key, quote_header_value(value,
-                                allow_token=key not in self._require_quoting))
-            for key, value in iteritems(self.options)
-        )))
-    __repr__ = __str__
-        
+
 ###############################################    UTILITIES, ENCODERS, PARSERS
 def get_environ_proxies():
     """Return a dict of environment proxies. From requests_."""
@@ -1190,11 +1140,6 @@ def get_environ_proxies():
     proxies = [(key, get_proxy(key + '_proxy')) for key in proxy_keys]
     return dict([(key, val) for (key, val) in proxies if val])
 
-def addslash(url):
-    '''Add a slash at the beginning of *url*.'''
-    if not url.startswith('/'):
-        url = '/%s' % url
-    return url
 
 def appendslash(url):
     '''Append a slash to *url* if it does not have one.'''
@@ -1202,16 +1147,18 @@ def appendslash(url):
         url = '%s/' % url
     return url
 
+
 def choose_boundary():
     """Our embarassingly-simple replacement for mimetools.choose_boundary."""
     return uuid4().hex
 
+
 def get_content_type(filename):
     return mimetypes.guess_type(filename)[0] or 'application/octet-stream'
 
+
 def encode_multipart_formdata(fields, boundary=None, charset=None):
-    """
-    Encode a dictionary of ``fields`` using the multipart/form-data mime format.
+    """Encode a dictionary of ``fields`` using the multipart/form-data format.
 
     :param fields:
         Dictionary of fields or list of (key, value) field tuples.  The key is
@@ -1234,7 +1181,7 @@ def encode_multipart_formdata(fields, boundary=None, charset=None):
         if isinstance(value, tuple):
             filename, data = value
             body.write(('Content-Disposition: form-data; name="%s"; '
-                        'filename="%s"\r\n' % (fieldname, filename))\
+                        'filename="%s"\r\n' % (fieldname, filename))
                        .encode(charset))
             body.write(('Content-Type: %s\r\n\r\n' %
                        (get_content_type(filename))).encode(charset))
@@ -1249,12 +1196,15 @@ def encode_multipart_formdata(fields, boundary=None, charset=None):
     content_type = 'multipart/form-data; boundary=%s' % boundary
     return body.getvalue(), content_type
 
+
 def hexmd5(x):
     return md5(to_bytes(x)).hexdigest()
 
+
 def hexsha1(x):
     return sha1(to_bytes(x)).hexdigest()
 
+
 def http_date(epoch_seconds=None):
     """
     Formats the time to match the RFC1123 date format as specified by HTTP
@@ -1268,6 +1218,7 @@ def http_date(epoch_seconds=None):
     """
     return formatdate(epoch_seconds, usegmt=True)
 
+
 #################################################################### COOKIE
 def create_cookie(name, value, **kwargs):
     """Make a cookie from underspecified parameters.
@@ -1300,28 +1251,35 @@ def create_cookie(name, value, **kwargs):
     result['path_specified'] = bool(result['path'])
     return Cookie(**result)
 
+
 def cookiejar_from_dict(cookie_dict, cookiejar=None):
     """Returns a CookieJar from a key/value dictionary.
 
     :param cookie_dict: Dict of key/values to insert into CookieJar.
     """
-    if cookiejar is None:
-        cookiejar = CookieJar()
-    if cookie_dict is not None:
-        for name in cookie_dict:
-            cookiejar.set_cookie(create_cookie(name, cookie_dict[name]))
-    return cookiejar
+    if not isinstance(cookie_dict, CookieJar):
+        if cookiejar is None:
+            cookiejar = CookieJar()
+        if cookie_dict is not None:
+            for name in cookie_dict:
+                cookiejar.set_cookie(create_cookie(name, cookie_dict[name]))
+        return cookiejar
+    else:
+        return cookie_dict
+
 
 def parse_cookie(cookie):
-    '''Parse an `HTTP cookie`_ string and return a dictionary of
-cookie name/values.'''
-    if cookie == '':
+    '''Parse an `HTTP cookie`_ string.
+
+    Return a dictionary of cookie name/values.
+    '''
+    if not cookie:
         return {}
     if not isinstance(cookie, BaseCookie):
         try:
             c = SimpleCookie()
             c.load(cookie)
-        except CookieError:
+        except CookieError:     # pragma    nocover
             # Invalid cookie
             return {}
     else:
@@ -1441,4 +1399,3 @@ class CacheControl(object):
                 headers.add_header('cache-control', 'proxy-revalidate')
         else:
             headers['cache-control'] = 'no-cache'
-
